//
// UR5FFR Si5351 NWT Lite
// v1.1
// Copyright (c) Andrey Belokon, 2017,2018 Odessa 
// https://github.com/andrey-belokon/
// GNU GPL license
//

#include "i2c.h"
#include "si5351a.h"

/*
  I2C address mapping
  I2C device found at address 0x60  ! Si5351
*/

// уровень сигнала на выходе Si5351. 0=2mA, 1=4mA, 2=6mA, 3=8mA
/*
 * CLK0 - Out
 * CLK1 - not used
 * CLK2 - SA612
 */
#define SI5351_CLK0_DRIVE   0
#define SI5351_CLK1_DRIVE   0
#define SI5351_CLK2_DRIVE   0

#define SI5351_CALIBRATION       25003566

#define IF_FREQ   455000
#define TRIPLE_FREQ   145000000

#define FIRMWARE_VERSION    "1.1 Lite"

Si5351 vfo5351;

char *ReadNumber(char *p, long *value)
{
  long val = 0;
  char c;
  bool negate;
  //if (negate = (*p == '-')) p++;
  while ((c = *p++) && isdigit(c)) val = val*10+c-'0';
  *value = val;//(negate ? -val : val);
  return p;
}

char *SkipWhitespace(char *p){
  while (*p && (*p == ' ' || *p == ','))
    p++;
  return p;
} 

char cmd_buf[100];
int icmd_buf = 0;
long freq;

bool UpdateFreq()
{
  if (freq < TRIPLE_FREQ)
    return vfo5351.set_freq(freq,0,freq+IF_FREQ);
  else
    return vfo5351.set_freq(freq/3,0,(freq+IF_FREQ)/3);
}

bool IsFreqOK()
{
  return vfo5351.is_freq_ok(0) && vfo5351.is_freq_ok(2);
}

void ParseCommand()
{
  char *p = cmd_buf;
  char cmd = *p++;
  long step,count,xfreq;
  int val;
  p = SkipWhitespace(p);
  switch (cmd) {
    case 'w':
      // get firmware version
      Serial.println(FIRMWARE_VERSION);
      break;
    case 'c':
      // калибровка частоты сишки
      vfo5351.set_freq(5000000,5000000,5000000);
      vfo5351.out_calibrate_freq();
      break;
    case 'x':
      // задать калибровочную частоту
      // x 25003566
      ReadNumber(p,&xfreq);
      vfo5351.set_xtal_freq(xfreq);
      break;
    case 'o':
      // off. отключить генерацию
      vfo5351.set_freq(0,0,0);
      break;
    case 'f':
      // установить частоту
      // f 5000000
      ReadNumber(p,&freq);
      if (IsFreqOK())
        UpdateFreq();
      else {
        UpdateFreq();
        delay(100);
        analogRead(A0);
      }
      break;
    case 'r':
      // read data
      // r step count
      p = ReadNumber(p,&step);
      p = SkipWhitespace(p);
      ReadNumber(p,&count);
      while (count-- > 0) {
        // при меньших значениях не успевает установиться частота на выходе
        /*if (UpdateFreq()) delay(10); 
        else delay(5);*/
        UpdateFreq();
        if (IsFreqOK()) {
          delay(10);
          val = analogRead(A0);
          val += analogRead(A0);
          val += analogRead(A0);
          val += analogRead(A0);
          val += analogRead(A0);
        } else {
          val = 0;
        }
        Serial.println(val);
        freq += step;
      }
      break;
  }
  icmd_buf = 0;
}

void loop()
{
  if (Serial.available()) {
    int inbyte = Serial.read();
  
    if (inbyte == 13){
      cmd_buf[icmd_buf] = 0;
      icmd_buf = 0;
      ParseCommand();
    } else if (icmd_buf < sizeof(cmd_buf)-1) {
      cmd_buf[icmd_buf++] = inbyte;
    }
  }
}

void setup()
{
  Serial.begin(115200);
  Serial.println(FIRMWARE_VERSION);
  i2c_init();
  // change for required output level
  vfo5351.setup(
    SI5351_CLK0_DRIVE,
    SI5351_CLK1_DRIVE,
    SI5351_CLK2_DRIVE
  ); 
  vfo5351.set_xtal_freq(SI5351_CALIBRATION);
  analogReference(INTERNAL);
  pinMode(A0,INPUT);
  //freq = 3000000;
  //UpdateFreq();
}


